package com.enteprise.matchAnalysis.service.impl;

import com.enteprise.base_data_month.domain.EnterpriseBaseDataMonth;
import com.enteprise.base_data_month.mapper.EnterpriseBaseDataMonthMapper;
import com.enteprise.base_data_year.domain.EnterpriseBaseDataYear;
import com.enteprise.base_data_year.mapper.EnterpriseBaseDataYearMapper;
import com.enteprise.enterprise.domain.Enterprise;
import com.enteprise.enterprise.dto.EnterpriseDto;
import com.enteprise.enterprise.mapper.EnterpriseMapper;
import com.enteprise.growthRate.domain.dto.GrowthRateDto;
import com.enteprise.industry_run.domain.EnterpriseBaseDataTotal;
import com.enteprise.industry_run.mapper.EnterpriseBaseDataTotalMapper;
import com.enteprise.matchAnalysis.domain.MatchAna;
import com.enteprise.matchAnalysis.domain.dto.MatchDto;
import com.enteprise.matchAnalysis.service.IMatchService;
import com.enteprise.season.domain.EnterpriseBaseDataSeason;
import com.enteprise.season.mapper.EnterpriseBaseDataSeasonMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

import static com.enteprise.util.CommonUtil.getRange;
import static com.enteprise.util.CommonUtil.sortByNumberInName;

@Service
public class MatchServiceImpl implements IMatchService {
    @Autowired
    private EnterpriseMapper enterpriseMapper;
    @Autowired
    private EnterpriseBaseDataMonthMapper enterpriseBaseDataMonthMapper;
    @Autowired
    private EnterpriseBaseDataSeasonMapper enterpriseBaseDataSeasonMapper;
    @Autowired
    private EnterpriseBaseDataYearMapper enterpriseBaseDataYearMapper;
    @Autowired
    private EnterpriseBaseDataTotalMapper enterpriseBaseDataTotalMapper;

    @Override
    public List<MatchDto> matchAnalysisList(MatchAna matchAna) {
        ArrayList<MatchDto> list = new ArrayList<>();
        //评估类型
        String dataKey = matchAna.getDataKey();
        //阈值
        double threshold = matchAna.getThreshold();
        //时间纬度
        int year = matchAna.getYear();
        int elecStartMonth = matchAna.getElecStartMonth();
        int elecEndMonth = matchAna.getElecEndMonth();
        int manStartSeason = matchAna.getManStartSeason();
        int manEndSeason = matchAna.getManEndSeason();
        //通过分类、行业代码过滤企业列表
        Enterprise enterprise = new Enterprise();
        enterprise.setCode(matchAna.getCode());
        enterprise.setTypeNum(matchAna.getTypeNum());
        List<EnterpriseDto> enterpriseDtos = sortByNumberInName(enterpriseMapper.selectAllWithType(enterprise));
        //对应评估类型业务逻辑
        if (dataKey.equals("powerConsume")){
            //用电逻辑,月度指标
            //每个企业从开始月份循环相加到结束月份
            List<Integer> monthRange = getRange(elecStartMonth, elecEndMonth);
            for (EnterpriseDto enterpriseDto : enterpriseDtos) {
                double powerConsume = 0;
                double oldPowerConsume = 0;
                double totalIndustrial = 0;
                double oldTotalIndustrial = 0;
                String enterpriseName = enterpriseDto.getEnterpriseName();
                for (int nowMonth : monthRange) {
                    //计算对应月份范围数值
                    EnterpriseBaseDataMonth dataMonth = new EnterpriseBaseDataMonth();
                    dataMonth.setEnterpriseName(enterpriseName);
                    dataMonth.setMonth(String.valueOf(nowMonth));
                    dataMonth.setYear(String.valueOf(year));
                    //获取今年和去年对应月份
                    List<EnterpriseBaseDataMonth> dataMonths = enterpriseBaseDataMonthMapper.selectEnterpriseBaseDataMonthList(dataMonth);
                    dataMonth.setYear(String.valueOf(year - 1));
                    List<EnterpriseBaseDataMonth> oldDataMonths = enterpriseBaseDataMonthMapper.selectEnterpriseBaseDataMonthList(dataMonth);
                    //计算今年和去年的电力和工业总产
                    if (!dataMonths.isEmpty() && !oldDataMonths.isEmpty()) {
                        EnterpriseBaseDataMonth dataMonth1 = dataMonths.get(0);
                        powerConsume += dataMonth1.getPowerConsume();
                        totalIndustrial += dataMonth1.getTotalIndustrialValue();
                        EnterpriseBaseDataMonth dataMonth2 = oldDataMonths.get(0);
                        oldPowerConsume += dataMonth2.getPowerConsume();
                        oldTotalIndustrial += dataMonth2.getTotalIndustrialValue();
                    }
                }
                //计算同比增速:（今年-去年）/去年
                double trueThreshold = trueThreshold(powerConsume, oldPowerConsume, totalIndustrial, oldTotalIndustrial);
                if (trueThreshold !=0 && trueThreshold < threshold) {
                    //初始化返回实体类并添加到list
                    MatchDto dto = new MatchDto(enterpriseName, trueThreshold, enterpriseDto.getLocation(), enterpriseDto.getCode(), enterpriseDto.getTypeNum(), enterpriseDto.getTypeName());
                    list.add(dto);
                }
            }
            return list;
        }
        if (dataKey.equals("employeeNumber")){
            //用人逻辑，季度指标
            List<Integer> seasonRange = getRange(manStartSeason, manEndSeason);
            for (EnterpriseDto enterpriseDto : enterpriseDtos){
                EnterpriseBaseDataTotal dataSeason = new EnterpriseBaseDataTotal();
                double employeeNumber = 0;
                double oldEmployeeNumber = 0;
                double totalIndustrial = 0;
                double oldTotalIndustrial = 0;
                String enterpriseName = enterpriseDto.getEnterpriseName();
                for (int nowSeason : seasonRange) {
                    dataSeason.setEnterpriseName(enterpriseName);
                    dataSeason.setSeason(String.valueOf(nowSeason));
                    dataSeason.setYear(String.valueOf(year));
                    //季度指标获取
                    List<EnterpriseBaseDataTotal> dataSeasons = enterpriseBaseDataTotalMapper.selectEnterpriseBaseDataTotalList(dataSeason);
                    dataSeason.setYear(String.valueOf(year - 1));
                    List<EnterpriseBaseDataTotal> oldDataSeasons = enterpriseBaseDataTotalMapper.selectEnterpriseBaseDataTotalList(dataSeason);
                    if (!dataSeasons.isEmpty()) {
                        for (EnterpriseBaseDataTotal item : dataSeasons){
                            employeeNumber += item.getEmployeeNumber();
                            totalIndustrial += item.getTotalIndustrialValue();
                        }
                    }
                    if (!oldDataSeasons.isEmpty()){
                        for (EnterpriseBaseDataTotal item : oldDataSeasons){
                            oldEmployeeNumber += item.getEmployeeNumber();
                            oldTotalIndustrial += item.getTotalIndustrialValue();
                        }
                    }
                }
                //计算同比增速:（今年-去年）/去年
                double trueThreshold =  trueThreshold(employeeNumber, oldEmployeeNumber, totalIndustrial, oldTotalIndustrial);
                if (trueThreshold !=0 && trueThreshold < threshold ) {
                    //初始化返回实体类并添加到list
                    MatchDto dto = new MatchDto(enterpriseName, trueThreshold, enterpriseDto.getLocation(), enterpriseDto.getCode(), enterpriseDto.getTypeNum(), enterpriseDto.getTypeName());
                    list.add(dto);
                }
            }
            return list;
        }
        if (dataKey.equals("energyConsume")){
            //用气逻辑，年度指标
            for (EnterpriseDto enterpriseDto : enterpriseDtos){
                EnterpriseBaseDataYear dataYear = new EnterpriseBaseDataYear();
                double energyConsume = 0;
                double oldEnergyConsume = 0;
                double totalIndustrial = 0;
                double oldTotalIndustrial = 0;
                String enterpriseName = enterpriseDto.getEnterpriseName();
                //年度指标获取
                dataYear.setEnterpriseName(enterpriseName);
                dataYear.setYear(String.valueOf(year));
                List<EnterpriseBaseDataYear> dataYears = enterpriseBaseDataYearMapper.selectEnterpriseBaseDataYearList(dataYear);
                dataYear.setYear(String.valueOf(year - 1));
                List<EnterpriseBaseDataYear> oldDataYears = enterpriseBaseDataYearMapper.selectEnterpriseBaseDataYearList(dataYear);
                if (!dataYears.isEmpty() && !oldDataYears.isEmpty()) {
                    EnterpriseBaseDataYear dataYear1 = dataYears.get(0);
                    energyConsume = dataYear1.getEnergyConsume();
                    totalIndustrial = dataYear1.getTotalIndustrialValue();
                    EnterpriseBaseDataYear dataYear2 = oldDataYears.get(0);
                    oldEnergyConsume = dataYear2.getEnergyConsume();
                    oldTotalIndustrial = dataYear2.getTotalIndustrialValue();
                }
                //计算同比增速:（今年-去年）/去年
                double trueThreshold =  trueThreshold(energyConsume, oldEnergyConsume, totalIndustrial, oldTotalIndustrial);
                if (trueThreshold !=0 && trueThreshold < threshold) {
                    //初始化返回实体类并添加到list
                    MatchDto dto = new MatchDto(enterpriseName, trueThreshold, enterpriseDto.getLocation(), enterpriseDto.getCode(), enterpriseDto.getTypeNum(), enterpriseDto.getTypeName());
                    list.add(dto);
                }
            }
            return list;
        }
        return list;
    }

    private double trueThreshold(double newValue, double oldValue,double newValue1, double oldValue1){
        double employeeRate = 0;
        double industrialRate = 0;
        if (oldValue != 0) {
            employeeRate = (newValue - oldValue) / oldValue;
        }
        if (oldValue1 != 0) {
            industrialRate = (newValue1 - oldValue1) / oldValue1;
        }
        return  employeeRate - industrialRate;
    }
}
